# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class FluxSuggestions(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'funcs': 'list[FluxSuggestion]'
    }

    attribute_map = {
        'funcs': 'funcs'
    }

    def __init__(self, funcs=None):  # noqa: E501,D401,D403
        """FluxSuggestions - a model defined in OpenAPI."""  # noqa: E501
        self._funcs = None
        self.discriminator = None

        if funcs is not None:
            self.funcs = funcs

    @property
    def funcs(self):
        """Get the funcs of this FluxSuggestions.

        :return: The funcs of this FluxSuggestions.
        :rtype: list[FluxSuggestion]
        """  # noqa: E501
        return self._funcs

    @funcs.setter
    def funcs(self, funcs):
        """Set the funcs of this FluxSuggestions.

        :param funcs: The funcs of this FluxSuggestions.
        :type: list[FluxSuggestion]
        """  # noqa: E501
        self._funcs = funcs

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, FluxSuggestions):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other
