# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class TemplateSummaryErrors(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'kind': 'TemplateKind',
        'reason': 'str',
        'fields': 'list[str]',
        'indexes': 'list[int]'
    }

    attribute_map = {
        'kind': 'kind',
        'reason': 'reason',
        'fields': 'fields',
        'indexes': 'indexes'
    }

    def __init__(self, kind=None, reason=None, fields=None, indexes=None):  # noqa: E501,D401,D403
        """TemplateSummaryErrors - a model defined in OpenAPI."""  # noqa: E501
        self._kind = None
        self._reason = None
        self._fields = None
        self._indexes = None
        self.discriminator = None

        if kind is not None:
            self.kind = kind
        if reason is not None:
            self.reason = reason
        if fields is not None:
            self.fields = fields
        if indexes is not None:
            self.indexes = indexes

    @property
    def kind(self):
        """Get the kind of this TemplateSummaryErrors.

        :return: The kind of this TemplateSummaryErrors.
        :rtype: TemplateKind
        """  # noqa: E501
        return self._kind

    @kind.setter
    def kind(self, kind):
        """Set the kind of this TemplateSummaryErrors.

        :param kind: The kind of this TemplateSummaryErrors.
        :type: TemplateKind
        """  # noqa: E501
        self._kind = kind

    @property
    def reason(self):
        """Get the reason of this TemplateSummaryErrors.

        :return: The reason of this TemplateSummaryErrors.
        :rtype: str
        """  # noqa: E501
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Set the reason of this TemplateSummaryErrors.

        :param reason: The reason of this TemplateSummaryErrors.
        :type: str
        """  # noqa: E501
        self._reason = reason

    @property
    def fields(self):
        """Get the fields of this TemplateSummaryErrors.

        :return: The fields of this TemplateSummaryErrors.
        :rtype: list[str]
        """  # noqa: E501
        return self._fields

    @fields.setter
    def fields(self, fields):
        """Set the fields of this TemplateSummaryErrors.

        :param fields: The fields of this TemplateSummaryErrors.
        :type: list[str]
        """  # noqa: E501
        self._fields = fields

    @property
    def indexes(self):
        """Get the indexes of this TemplateSummaryErrors.

        :return: The indexes of this TemplateSummaryErrors.
        :rtype: list[int]
        """  # noqa: E501
        return self._indexes

    @indexes.setter
    def indexes(self, indexes):
        """Set the indexes of this TemplateSummaryErrors.

        :param indexes: The indexes of this TemplateSummaryErrors.
        :type: list[int]
        """  # noqa: E501
        self._indexes = indexes

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, TemplateSummaryErrors):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other
